/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_1
*/
#include <stdio.h>
#include <smgdef.h>
#include <ssdef.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

/*
**  add vertical bars to the display, joining up sisters
*/
add_sister_bars(ptr, level)
LINE *ptr;
int level;
{
    ptr->connections[level-1] |= C_UP | C_RIGHT;
    add_sister_bars_1(ptr->back_ptr, level);
}

add_sister_bars_1(ptr, level)
LINE *ptr;
int level;
{
    if (ptr->n_ptr[level])
    {
        ptr->connections[level-1] |= C_DOWN;
    }
    else
    {
        ptr->connections[level-1] |= C_DOWN | C_UP;
        add_sister_bars_1(ptr->back_ptr, level);
    }
}

refresh_display()
{
    char aline[1024];
    int i, status;
    LINE *ptr;

    smg$begin_pasteboard_update(&paste);
    /*
    **  set ptr to point to line #current_line
    */
    ptr = first_line_ptr;
    for (i = 0; i < current_top_line; i++)
        ptr = ptr->forw_ptr;

    smg$set_cursor_abs(&display1, &1, &1);
    /*
    **  throw out to display1 what should be there
    */
    for (i = 1; i <= DISPLAY1_ROWS; i++)
    {
        if (current_top_line + i - 1 < total_lines)
        {
            /*
            ** show this line - line i, no scroll, hence no direction
            */
            display_line(ptr,i,0,0);
            ptr = ptr->forw_ptr;
        }
        else
        {
            /*
            **  delete to the end of the screen
            */
            smg$erase_display(&display1,&i,&1);
            break;
        }
    }
    /*
    **  now go and set the node upon which the cursor currently resteth
    **  with a special rendition
    */
    render(current_line,current_level,SPECIAL_RENDITION);
    rest_cursor();
    smg$end_pasteboard_update(&paste);
}

/*
**  change the rendition of a displayed directory
*/
render(line,level,rendition)
int line, level, rendition;
{
    int i,x,y,r,c,status;
    LINE *temp;

    temp = first_line_ptr;
    for (i = 0; i < line; i++)
        temp = temp->forw_ptr;

    y = line - current_top_line + 1;
    x = LEVEL_OFFSET(level);
    r = 1;
    c = NAME_SHORT_SIZE;

    /*
    **  if the rendition is NORMAL_RENDITION
    **  we'll change to UNDERLINE_RENDITION if necessary - i.e. if
    **  there is something below me and i don't have a sister
    */

    /*
    **  don't waste time on fields not currently displayed
    */
    if (y > 0 && y <= DISPLAY1_ROWS)
    {
        if (rendition == NORMAL_RENDITION &&
            !temp->n_ptr[level]->sister &&
            temp->forw_ptr &&
            temp->forw_ptr->n_ptr[level])
            rendition = UNDERLINE_RENDITION;

        /*
        **  do this via pasteboard updates, to stop cursor flying around
        */
        smg$begin_pasteboard_update(&paste);
        status = smg$change_rendition(&display1,&y,&x,&r,&c,&rendition);
        if (status != SS$_NORMAL && status != SMG$_INVROW)
            panic(status,"changing rendition","render");

        /*
        **  make sure cursor remains at bottom of screen
        */
        rest_cursor();

        /*
        **  now update display
        */
        smg$end_pasteboard_update(&paste);
    }
}

rest_cursor()
{
    /*
    **  a little function to lay the cursor to rest at the bottom
    **  right-hand corner of the screen
    */
    int row, column;

    row = CUR_REST_Y;
    column = CUR_REST_X;
    smg$set_cursor_abs(&displayb, &row, &column);
}
